#! /usr/bin/env python

from openturns import *

TESTPREAMBLE()
RandomGenerator().SetSeed(0)

try :
    # Instanciate one distribution object
    dimension = 3
    meanPoint = NumericalPoint(dimension, 1.0)
    meanPoint[0] = 0.5
    meanPoint[1] = -0.5
    sigma = NumericalPoint(dimension, 1.0)
    sigma[0] = 2.0
    sigma[1] = 3.0

    sample = NumericalSample(0, dimension)
    # Create a collection of distribution
    aCollection = DistributionCollection()

    aCollection.add( Distribution(Normal(meanPoint, sigma, IdentityMatrix(dimension))) )
    sample.add(meanPoint)
    meanPoint += NumericalPoint(meanPoint.getDimension(), 1.0)
    aCollection.add( Distribution(Normal(meanPoint, sigma, IdentityMatrix(dimension)) ))
    sample.add(meanPoint)
    meanPoint += NumericalPoint(meanPoint.getDimension(), 1.0)
    aCollection.add( Distribution(Normal(meanPoint, sigma, IdentityMatrix(dimension)) ))
    sample.add(meanPoint)

    # Instanciate one distribution object
    distribution = KernelMixture(Normal(), sigma, sample)
    print "Distribution " , distribution
    distributionRef = Mixture(aCollection)

    # Is this distribution elliptical ?
    print "Elliptical = ", distribution.isElliptical()

    # Is this distribution continuous ?
    print "Continuous = ", distribution.isContinuous()

    # Test for realization of distribution
    oneRealization = distribution.getRealization()
    print "oneRealization=", oneRealization 
 
    # Test for sampling
    size = 10000
    oneSample = distribution.getNumericalSample( size )
    print "oneSample first=" , oneSample[0] , " last=" , oneSample[size - 1] 
    print "mean=" , oneSample.computeMean() 
    print "covariance=" , oneSample.computeCovariance()

    # Define a point
    point = NumericalPoint( distribution.getDimension() , 1.0)
    print "Point= " , point 

    # Show PDF and CDF of point
    eps = 1e-5
        
    # derivative of PDF with regards its arguments
    DDF = distribution.computeDDF( point )
    print "ddf     =" , DDF
    print "ddf (ref)=", distributionRef.computeDDF( point )
    # by the finite difference technique
    ddfFD = NumericalPoint(dimension)
    for i in range(dimension) :
        left = NumericalPoint(point)
        left[i] += eps
        right = NumericalPoint(point)
	right[i] -= eps
	ddfFD[i] = (distribution.computePDF(left) - distribution.computePDF(right)) / (2.0 * eps)
    print "ddf (FD)=", ddfFD

    # PDF value
    PDF = distribution.computePDF( point )
    print "pdf     =%.6f" % PDF 
    print "pdf (ref)=%.6f" % distributionRef.computePDF( point )
    # by the finite difference technique from CDF
    if (dimension == 1) :
        print "pdf (FD)=%.6f" % ((distribution.computeCDF( point + NumericalPoint(1, eps) ) - distribution.computeCDF( point  + NumericalPoint(1, -eps) )) / (2.0 * eps))

    # derivative of the PDF with regards the parameters of the distribution
    CDF = distribution.computeCDF( point )
    print "cdf=%.6f" % CDF
    print "cdf (ref)=%.6f" % distributionRef.computeCDF( point )

    # quantile
    quantile = distribution.computeQuantile( 0.95 )
    print "quantile=" , quantile 
    print "quantile (ref)=" , distributionRef.computeQuantile( 0.95 )
    print "cdf(quantile)=%.6f" % distribution.computeCDF(quantile) 
    mean = distribution.getMean()
    print "mean=" , mean 
    print "mean (ref)=" , distributionRef.getMean()
    covariance = distribution.getCovariance()
    print "covariance=" , covariance 
    print "covariance (ref)=" , distributionRef.getCovariance()
    #parameters = distribution.getParametersCollection()
    #print "parameters=" , parameters 
                        
except :
  import sys
  print "t_Mixture_std.py", sys.exc_type, sys.exc_value

